@@ -1,4 +1,3 @@
-#!/usr/bin/env python
 import os
 import time
 import datetime
@@ -88,9 +87,9 @@ class SGEStats(object):
 
     def job_multiply(self, hash):
         """
-        this function deals with sge jobs with a task range, ie qsub -t 1-20:1
-        makes 20 jobs. self.jobs needs to represent that it is 20 jobs instead
-        of just 1.
+        This function deals with sge jobs with a task range.  For example,
+        'qsub -t 1-20:1' makes 20 jobs. self.jobs needs to represent that it is
+        20 jobs instead of just 1.
         """
         sz_range = hash['tasks']
         dashpos = sz_range.find('-')
@@ -495,7 +494,7 @@ class SGELoadBalancer(LoadBalancer):
         it will feed these stats to SGEStats, which parses the XML.
         it will return two arrays: one of hosts, each host has a hash with its
         host information inside. The job array contains a hash for every job,
-        containing statistics about the job name, priority, etc
+        containing statistics about the job name, priority, etc.
         """
         log.debug("starting get_stats")
         master = self._cluster.master_node
@@ -519,7 +518,7 @@ class SGELoadBalancer(LoadBalancer):
                                                  ignore_exit_status=True,
                                                  source_profile=True))
         except Exception, e:
-            log.error("Error occured getting SGE stats via ssh. "
+            log.error("Error occurred getting SGE stats via ssh. "
                       "Cluster terminated?")
             log.error(e)
             return -1
